﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TIS.Imaging;

namespace Image_And_AVI_Capture
{
    public partial class Form1 : Form
    {

        private bool m_bFitImageToWindow = true;
        private FrameFilter _saveImageFrameFilter;
        private int imageCounter = 0;
        private MediaStreamSink _aviSink;
 
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            _aviSink = new MediaStreamSink();
            //
            foreach (AviCompressor codec in AviCompressor.AviCompressors)
            {
                comboBox1.Items.Add(codec);
                if (codec.Name == "DV Video Encoder")
                {
                    _aviSink.Codec = codec;
                    comboBox1.SelectedIndex = comboBox1.FindStringExact("DV Video Encoder");

                }
            }

            _saveImageFrameFilter = FrameFilter.Create("Save Image");
            if (_saveImageFrameFilter == null)
            {
                MessageBox.Show($"Failed to load the \"Save Image\" frame filter!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            icImagingControl1.DeviceFrameFilters.Add(_saveImageFrameFilter);

            cboMediaStreamContainer.DataSource = TIS.Imaging.MediaStreamContainer.MediaStreamContainers;

            try
            {
                icImagingControl1.LoadDeviceStateFromFile("device.xml", true);
                _aviSink.Filename = System.IO.Path.ChangeExtension($"{icImagingControl1.Device.ToString()}.avi", CurrentMediaStreamContainer.PreferredFileExtension);
            }
            catch
            {
                // xmlファイルの読み込み、接続したカメラを開くのに失敗した時の処理
            }

            if (m_bFitImageToWindow)
            {
                // ライブ表示のサイズをフォームのサイズに調整する
                icImagingControl1.LiveDisplayDefault = false;
                icImagingControl1.LiveDisplaySize = icImagingControl1.Size;
            }

            //GUI上のボタンの有効化/無効化を制御
            UpdateControls();

        }

        /// <summary>
        /// アプリ終了時にライブスタートを停止する。
        /// </summary>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (icImagingControl1.DeviceValid)
            {
                icImagingControl1.LiveStop();
            }
        }

        /// <summary>
        /// ライブ表示のサイズをフォームのサイズに調整する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            if (m_bFitImageToWindow)
            {
                if (icImagingControl1.DeviceValid)
                {
                    icImagingControl1.LiveDisplaySize = icImagingControl1.Size;
                }
            }
        }

        /// <summary>
        /// デバイスダイアログ画面を表示する。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdDevice_Click(object sender, EventArgs e)
        {
            OpenNewVideoCaptureDevice();
        }

        /// <summary>
        /// デバイスプロパティ画面を表示する。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdProperties_Click(object sender, EventArgs e)
        {
            ShowDeviceProperties();
        }

        /// <summary>
        /// StartボタンとStopボタンの入れ替え
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdLive_Click(object sender, EventArgs e)
        {
            if (icImagingControl1.LiveVideoRunning)
            {
                StopLiveVideo();
            }
            else
            {
                StartLiveVideo();
            }
        }

        ////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////

        /// <summary>
        /// ライブスタートを開始する
        /// </summary>
        private void StartLiveVideo()
        {
            _aviSink=new TIS.Imaging.MediaStreamSink(CurrentMediaStreamContainer, CurrentVideoCodec, System.IO.Path.ChangeExtension($"{icImagingControl1.Device.ToString()}.avi", CurrentMediaStreamContainer.PreferredFileExtension) );
            _aviSink.SinkModeRunning = !checkBoxAVIPause.Checked;
            icImagingControl1.Sink = _aviSink;
            icImagingControl1.LiveStart();
            //GUI上のボタンの有効化/無効化を制御
            UpdateControls();
        }

        /// <summary>
        /// ライブスタートを停止する。
        /// </summary>
        private void StopLiveVideo()
        {
            icImagingControl1.LiveStop();
            //GUI上のボタンの有効化/無効化を制御
            UpdateControls();
        }

        private void UpdateControls()
        {
            cmdLive.Enabled = icImagingControl1.DeviceValid;
            cmdProperties.Enabled = icImagingControl1.DeviceValid;
            buttonAVIProperties.Enabled = !icImagingControl1.LiveVideoRunning;
            cboMediaStreamContainer.Enabled = !icImagingControl1.LiveVideoRunning;
            comboBox1.Enabled = !icImagingControl1.LiveVideoRunning;
            if (icImagingControl1.LiveVideoRunning)
            {
                cmdLive.Text = "Stop Live";
            }
            else
            {
                cmdLive.Text = "Start Live";
            }
        }

        /// <summary>
        /// デバイスダイアログ画面の表示をする。
        /// </summary>
        private void OpenNewVideoCaptureDevice()
        {
            if (icImagingControl1.DeviceValid)
            {
                StopLiveVideo();
            }
            else
            {
                icImagingControl1.Device = "";
            }
            icImagingControl1.ShowDeviceSettingsDialog();
            cmdLive.Enabled = icImagingControl1.DeviceValid;
            cmdProperties.Enabled = icImagingControl1.DeviceValid;
            if (icImagingControl1.DeviceValid)
            {
                // xmlファイルにデバイス情報を保存し、次回プログラム実行時に自動的に開くようにする。
                icImagingControl1.SaveDeviceStateToFile("device.xml");
                _aviSink.Filename = System.IO.Path.ChangeExtension($"{icImagingControl1.Device.ToString()}.avi", CurrentMediaStreamContainer.PreferredFileExtension);
            }
        }

        /// <summary>
        /// プロパティ設定画面を表示する。
        /// プロパティ設定画面で【device.xml】に設定した内容ををすべて保存する。
        /// </summary>
        private void ShowDeviceProperties()
        {
            if (icImagingControl1.DeviceValid)
            {
                icImagingControl1.ShowPropertyDialog();
                icImagingControl1.SaveDeviceStateToFile("device.xml");
            }
        }


        /// <summary>
        /// デバイスロストしたときに発生すイベントハンドラーです。
        /// </summary>
        private void DeviceLost()
        {
            MessageBox.Show("Device Lost!");
            //GUI上のボタンの有効化/無効化を制御
            UpdateControls();
        }

        private void icImagingControl1_DeviceLost(object sender, TIS.Imaging.ICImagingControl.DeviceLostEventArgs e)
        {
            // デバイスロスト中、ダイアログ画面を表示してても（メソッドの実行が完了していなくても）、制御が戻ってくるようにするためBeginInvoke処理を追加しています。
            BeginInvoke(new Action(() => DeviceLost()));
        }

        private void buttonSnap_Click(object sender, EventArgs e)
        {
            if(_saveImageFrameFilter != null && icImagingControl1.DeviceValid && icImagingControl1.LiveVideoRunning)
            {
                imageCounter++;
                string imageFileName = String.Format("Image{0}.jpg",imageCounter);
                _saveImageFrameFilter.SetStringParameter("ImageName", imageFileName);
            }
        }

        private void checkBoxAVIPause_CheckedChanged(object sender, EventArgs e)
        {
            icImagingControl1.Sink.SinkModeRunning = !icImagingControl1.Sink.SinkModeRunning;
        }

        private void buttonAVIProperties_Click(object sender, EventArgs e)
        {
            try
            {
                _aviSink.Codec.ShowPropertyPage();
            } catch 
            {

            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {

            foreach (AviCompressor codec in AviCompressor.AviCompressors)
            {
                if (codec.Name == comboBox1.SelectedItem.ToString())
                {
                    _aviSink.Codec = codec;
                }
            }
        }

        private void cboMediaStreamContainer_SelectedIndexChanged(object sender, EventArgs e)
        {
            fillCodecListItems();

            buttonAVIProperties.Enabled = CurrentVideoCodec == null ? false : CurrentVideoCodec.PropertyPageAvailable;
        }


        private void fillCodecListItems()
        {
            var cont = CurrentMediaStreamContainer;
            if (cont != null)
            {
                comboBox1.DataSource = TIS.Imaging.AviCompressor.AviCompressors.Where(p => cont.IsCodecSupported(p)).ToList();
            }
            else
            {
                comboBox1.DataSource = TIS.Imaging.AviCompressor.AviCompressors;
            }
        }
        private TIS.Imaging.MediaStreamContainer CurrentMediaStreamContainer
        {
            get
            {
                return (TIS.Imaging.MediaStreamContainer)cboMediaStreamContainer.SelectedItem;
            }
        }

        private TIS.Imaging.AviCompressor CurrentVideoCodec
        {
            get
            {
                var codec = (TIS.Imaging.AviCompressor)comboBox1.SelectedItem;
                if (codec == null)
                {
                    return null;
                }

                if (CurrentMediaStreamContainer != null && CurrentMediaStreamContainer.IsCodecSupported(codec))
                {
                    return codec;
                }
                else
                {
                    return null;
                }
            }
        }
    }
}
